package com.zsc.edu.gateway.modules.iot.product.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zsc.edu.gateway.exception.ConstraintException;
import com.zsc.edu.gateway.modules.iot.product.dto.ProductDto;
import com.zsc.edu.gateway.modules.iot.product.entity.Product;
import com.zsc.edu.gateway.modules.iot.product.mapper.ProductMapper;
import com.zsc.edu.gateway.modules.iot.product.query.ProductQuery;
import com.zsc.edu.gateway.modules.iot.product.repo.ProductRepository;
import com.zsc.edu.gateway.modules.iot.product.service.ProductService;
import com.zsc.edu.gateway.modules.iot.tsl.dto.ParamDto;
import com.zsc.edu.gateway.modules.iot.tsl.entity.Param;
import com.zsc.edu.gateway.modules.iot.tsl.mapper.ParamMapper;
import com.zsc.edu.gateway.modules.iot.tsl.repo.ParamRepository;
import com.zsc.edu.gateway.modules.iot.tsl.service.ParamService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;


/**
 * @author zhaung
 */
@AllArgsConstructor
@Service
public class ProductServiceImpl extends ServiceImpl<ProductRepository, Product> implements ProductService {
    private final ProductMapper mapper;
    private final ParamMapper paramMapper;
    private final ParamRepository paramRepo;
    private final ParamService paramService;

    /**
     * 新建产品
     */
    @Override
    public Product create(ProductDto dto) {
        if (baseMapper.findByName(dto.getName()) != null) {
            throw new ConstraintException("该设备已存在！");
        }
        Product product = mapper.toEntity(dto);
        save(product);
        paramService.create(dto.getParams(), product.getId(), Param.ForeignType.PRODUCT);
        return product;
    }
    /**
     * 更新产品
     */
    @Override
    public Product update(ProductDto dto, Long id) {
        Product product = baseMapper.selectById(id);
        mapper.convert(dto, product);
        updateById(product);
        paramService.update(dto.getParams(), product.getId());
        return product;
    }

    //    /**
//     * 分页查询产品
//     *
//     * @param query    查询表单
//     * @param page  分页参数
//     */
//    @Override
//    public Page<Product> page(Page<Product> page, ProductQuery query) {
//        return baseMapper.page(page, query.wrapper());
//    }
    @Override
    public Product detail(Long id) {
        return baseMapper.selectById(id);
    }

    /**
     * 删除
     */
    @Override
    public boolean delete(Long id) {
        removeById(id);
        paramService.delete(id);
        return true;
    }
}
