package com.zsc.edu.gateway.modules.iot.tsl.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.zsc.edu.gateway.modules.iot.tsl.dto.EventDto;
import com.zsc.edu.gateway.modules.iot.tsl.entity.Event;
import com.zsc.edu.gateway.modules.iot.tsl.query.EventQuery;
import com.zsc.edu.gateway.modules.iot.tsl.service.EventService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;


/**
 * @author zhuang
 */
@AllArgsConstructor
@RestController
@RequestMapping("api/rest/tsl/event")
public class EventController {

    private final EventService service;


    /**
     * 创建事件
     *
     * @param dto 创建的事件
     */
    @PostMapping
    public Event create(@RequestBody EventDto dto) {
        return service.create(dto);
    }


    /**
     * 更新事件
     *
     * @return 更新后的事件
     */
    @PatchMapping("{id}")
    public Event update(@RequestBody EventDto dto, @PathVariable("id") Long id) {
        return service.update(dto, id);
    }


    /**
     * 查询事件
     *
     * @param query    事件的查询表单
     * @param page 分页参数
     * @return Page<Device> 事件分页数据
     */
    @GetMapping
    public Page<Event> page(Page<Event> page, EventQuery query) {
        return service.page(page, query.wrapper());
    }


    /**
     * 删除事件
     *
     * @param id 事件编号
     * @return 被删除的事件
     */
    @DeleteMapping("{id}")
    public Boolean delete(@PathVariable("id") Long id) {
        return service.delete(id);
    }

    /**
     * 事件
     * 查询详情
     *
     * @param id ID
     * @return 任务
     */
    @GetMapping("{id}")
    public Event detail(@PathVariable("id") Long id) {
        return service.detail(id);
    }


}
