package com.zsc.edu.gateway.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.zsc.edu.gateway.domain.ParamBuilder;
import com.zsc.edu.gateway.exception.ConstraintException;
import com.zsc.edu.gateway.modules.iot.tsl.dto.ParamDto;
import com.zsc.edu.gateway.modules.iot.tsl.entity.DataType;
import com.zsc.edu.gateway.modules.iot.tsl.entity.Param;
import com.zsc.edu.gateway.modules.iot.tsl.repo.ParamRepository;
import com.zsc.edu.gateway.modules.iot.tsl.service.ParamService;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import static org.junit.jupiter.api.Assertions.*;

@SpringBootTest
public class ParamServiceTest {
    @Autowired
    private ParamService service;
    @Autowired
    private ParamRepository repo;

    private Param param1;
    private Param param2;

    @BeforeEach
    void setUp() {
        param1 = ParamBuilder.aParam().uint("测试1").build();
        repo.insert(param1);
        param2 = ParamBuilder.aParam().uint("测试2").build();
        repo.insert(param2);
    }

    @Test
    void list() {
        LambdaQueryWrapper<Param> queryWrapper = new LambdaQueryWrapper<>();
        assertEquals(2, service.list(queryWrapper.like(Param::getUint, "测试")).size());
        assertEquals(1, service.list(queryWrapper.eq(Param::getUint, param1.getUint())).size());
//        assertEquals(2, service.list().size());
    }

    @Test
    void createParams() {
        List<ParamDto> params = createParamDtoList();
        Long foreignId = 1L;
        Boolean result = service.create(params, foreignId, null);
        assertTrue(result);
        List<Param> insertedParams = service.list(new LambdaQueryWrapper<Param>().like(Param::getName, "PARAM_NAME"));
        assertEquals(2, insertedParams.size()); // 因为setUp方法已经插入了两个参数
    }

    @Test
    void updateParams() {
        ParamDto dto = new ParamDto();
        dto.setName("PARAM_NAME_UPDATE");
        dto.setIdentifier("测试更新");
        List<ParamDto> params = new ArrayList<>();
        params.add(dto);
        assertTrue(service.update(params, param2.getId()));

        // 确保获取到的 param 对象是最新的
        Param param = repo.selectById(param2.getId());
        assertNotNull(param); // 确保 param 不为空

        // 更新后的 identifier 应该是 "测试更新"
        assertEquals("测试更新", param.getIdentifier());
        assertEquals(param2.getId(), param.getId());
    }


    @AfterEach
    void tearDown() {
        repo.delete(new LambdaQueryWrapper<Param>()
                .in(Param::getId, param1.getId(), param2.getId())
                .or()
                .in(Param::getName, "PARAM_NAME_1", "PARAM_NAME_2", "PARAM_NAME_UPDATE"));
    }

    private List<ParamDto> createParamDtoList() {
        ParamDto paramDto1 = new ParamDto();
        paramDto1.setName("PARAM_NAME_1");
        paramDto1.setIdentifier("PARAM_VALUE_1");
        paramDto1.setDataType(DataType.DATE);
        ParamDto paramDto2 = new ParamDto();
        paramDto2.setName("PARAM_NAME_2");
        paramDto2.setIdentifier("PARAM_VALUE_2");
        paramDto2.setDataType(DataType.DATE);
        return Arrays.asList(paramDto1, paramDto2);
    }
}
